local device_type_id, item_list = ...

local _core = require("core")
local _logger = require("logging")

local gateway_id = _core.get_gateway().id

local devices = _core.get_devices()
local device_id = ""

if  devices then
    for _, dev in ipairs( devices ) do
        if dev.gateway_id == gateway_id and dev.device_type_id == device_type_id then
            _logger.debug("Device " .. device_type_id .. " is already present")
            device_id = dev.id
            break
        end
    end
end

if device_id == "" then
    _logger.info("Start device creation")
    device_id = loadfile("HUB:ezlo_protocols/scripts/devices/" .. device_type_id)()
end

local device_items = _core.get_items_by_device_id(device_id) or {}
local item_names = loadfile("HUB:ezlo_protocols/scripts/resources/" .. item_list)()

for _, item in ipairs(item_names) do
    local item_found = false

    for _, device_item in ipairs(device_items) do
        if device_item.name == item then
            _G.ezlo_protocols[item] = device_item.id
            item_found = true
            break
        end
    end

    if not item_found then
        local item_script =  loadfile("HUB:ezlo_protocols/scripts/items/" .. item)

        local success, errmsg_or_item_id = pcall(item_script, {
            device_id = device_id,
            action = "create"
        })

        if not success then
            _logger.error("Failed to set item: " .. item .. ", error: " .. errmsg_or_item_id)
            
            return
        else
            _G.ezlo_protocols[item] = errmsg_or_item_id
        end
    end
end

if device_type_id == "udp_manager" then
    local udp_setting_names = loadfile("HUB:ezlo_protocols/scripts/resources/udp_setting_names")()

    for _, setting in ipairs( udp_setting_names ) do    
        loadfile("HUB:ezlo_protocols/scripts/settings/" .. setting){
            device_id = device_id
        }
    end
elseif device_type_id == "http_manager" then
    local http_setting_names = loadfile("HUB:ezlo_protocols/scripts/resources/http_setting_names")()

    for _, setting in ipairs( http_setting_names ) do    
        loadfile("HUB:ezlo_protocols/scripts/settings/" .. setting){
            device_id = device_id
        }
    end
end

return device_id
