local _logger = require("logging")

function URL_ComponentEncode (url)
    url = url:gsub("([^%w_%%%-%.~])", function (c)
                            return string.format("%%%02X", string.byte(c))
                         end)

    return url
end

function GetTableFromString (string)
    local t = {}

    for x in string.gmatch(string, '([^&]+)') do
        for x, y in string.gmatch(x, "(.*)=(.*)") do
           t[x] = y
        end
    end

    return t
end

local ITEM_NAME = "http_client_send"

local function CreateItem (device_id)
    local _core = require("core")

    _logger.info("Create item: " .. ITEM_NAME)

    local item_id = _core.add_item {
        name = ITEM_NAME,
        has_getter = false,
        has_setter = true,
        show = true,
        value_type = "dictionary.string",
        value = {},
        device_id = device_id
    }
    
    if item_id then
        _logger.info("Created item ID: " .. item_id)
    else
        _logger.error("Failed to create item")
    end

    return item_id
end

local function SetItemValue (value)
    if not value.domain and not value.ip_address  then
        _logger.error("Neither IP address nor Domain are specified.")

        return
    end

    value.data = value.data or ""
    value.ip_address = value.ip_address or value.domain
    value.method = value.method or "GET"
    value.port = value.port or "80"
    value.path = value.path or "/"

    if value.path:sub(1, 1) ~= "/" then
        value.path = "/" .. value.path
    end

    if value.query_params then
        local t = GetTableFromString(value.query_params)

        local temp = ""
        for key, val in pairs(t) do
            temp = temp .. URL_ComponentEncode(key) .. "=" .. URL_ComponentEncode(val) .. "&"
        end

        value.query_params = "?" .. temp:sub(1, #temp - 1)
    else
        value.query_params = ""
    end

    if value.headers then
        value.headers  = GetTableFromString(value.headers)
    else
        value.headers = {}
    end

    local _http = require("http")

    local http_url = "http://" .. value.ip_address .. ":" .. value.port .. value.path .. value.query_params
    local request_id = _http.request( { url = http_url, type = value.method, data = value.data, headers = value.headers,  handler = "HUB:ezlo_protocols/scripts/events/http_events_handler" } )

    if not request_id then
        _logger.debug("Fail to connect via HTTP")
        
        return
    end
    
    _G.ezlo_protocols["http_request_id"] = request_id
    _G.ezlo_protocols[request_id] = value.data
end

----------------------------------------------------------------------
local params = ...

if params.action == "create" then

    return CreateItem(params.device_id)

elseif params.action == "set" then

    local http_request_cooldown = (loadfile("HUB:ezlo_protocols/scripts/functions/get_setting_by_name"){
        device_id = params.device_id,
        setting_name = "http_request_cooldown"
    } or {}).value or 1

    local time_elapsed = os.time() - _G.ezlo_protocols.http_last_request_time

    if time_elapsed > http_request_cooldown then
        _G.ezlo_protocols.http_last_request_time = os.time()

        return SetItemValue(params.value)
    end

end