local ITEM_NAME = "http_server_receive"
local HTTP_SERVER_URL_SETTING_NAME = "http_server_url"
local HTTP_ID_SETTING_NAME = "http_id"
local USE_HTTPS_SETTING_NAME = "use_https"
local ENABLE_HTTP_SERVER_SETTING_NAME = "enable_http_server"

local _core = require("core")

local function CreateItem (device_id)
    local _logger = require("logging")

    _logger.info("Create item: " .. ITEM_NAME)

    local item_id = _core.add_item {
        name = ITEM_NAME,
        has_getter = true,
        has_setter = false,
        show = true,
        value_type = "dictionary.string",
        value = {},
        device_id = device_id
    }
    
    if item_id then
        _logger.info("Created item ID: " .. item_id)
    else
        _logger.error("Failed to create item")
    end

    return item_id
end

local function EnableServer(device_id)
    local _http_server = require("http_server")
    local _logger = require("logging")

    local http_server_url_setting = loadfile("HUB:ezlo_protocols/scripts/functions/get_setting_by_name") {
        device_id = device_id,
        setting_name = HTTP_SERVER_URL_SETTING_NAME
    }

    local http_id_setting = loadfile("HUB:ezlo_protocols/scripts/functions/get_setting_by_name") {
        device_id = device_id,
        setting_name = HTTP_ID_SETTING_NAME
    }

    local use_https_setting = loadfile("HUB:ezlo_protocols/scripts/functions/get_setting_by_name") {
        device_id = device_id,
        setting_name = USE_HTTPS_SETTING_NAME
    }

    local success, url_info = pcall(_http_server.get_link_info, http_id_setting.value)

    if not success then
        _logger.info("Create server link for receiving events")

        success, url_info = pcall(_http_server.create_link_url,
            {
                link_id = http_id_setting.value,
                handler = "HUB:ezlo_protocols/scripts/events/http_server_events_handler",
            })

        if not success then
            _logger.error("Failed to create server link: " .. url_info)

            return
        end

        _core.set_setting_value(http_id_setting.id, url_info.link_id, "synced")

        local url = url_info.url

        if use_https_setting.value == true then
            url = url_info.secure_url
        end

        _core.set_setting_value(http_server_url_setting.id, url, "synced")

        _logger.debug("Created server link: " .. url) 

        _G.ezlo_protocols[ENABLE_HTTP_SERVER_SETTING_NAME] = url
    else
        _G.ezlo_protocols[ENABLE_HTTP_SERVER_SETTING_NAME] = "server_is_enable"
    end
end

local function DisableServer(device_id, action)
    local _http_server = require("http_server")
    local _logger = require("logging")

    local http_id_setting = loadfile("HUB:ezlo_protocols/scripts/functions/get_setting_by_name") {
        device_id = device_id,
        setting_name = HTTP_ID_SETTING_NAME
    }
    
    local value = http_id_setting.value

    if action == "restart" then
        value = _G.ezlo_protocols["HTTP_ID_CURRENT_VALUE"]
    end

    local success, errmsg = pcall(_http_server.delete_link_url, value)

    if not success then
        _logger.error("Failed to close HTTP server: " .. errmsg)
        return
    end

    local http_server_url_setting = loadfile("HUB:ezlo_protocols/scripts/functions/get_setting_by_name") {
        device_id = device_id,
        setting_name = HTTP_SERVER_URL_SETTING_NAME
    }

    if action == "disable" then
        _core.set_setting_value(http_id_setting.id, "", "synced")
    end

    _core.set_setting_value(http_server_url_setting.id, "", "synced")

    _logger.info("HTTP Server is disabled")
    
    _G.ezlo_protocols[ENABLE_HTTP_SERVER_SETTING_NAME] = nil
end

----------------------------------------------------------------------
local params = ...

if params.action == "create" then
    return CreateItem(params.device_id)

elseif params.action == "enable" then
    EnableServer(params.device_id)

elseif params.action == "disable" then
    DisableServer(params.device_id, params.action)

elseif params.action == "restart" then
    DisableServer(params.device_id, params.action)
    EnableServer(params.device_id)
end
