local _logger = require("logging")
local _core = require("core")

local ITEM_NAME = "udp_client_send"
local RELATED_SETTING_NAME = "send_as_broadcast"

local function CreateItem (device_id)
    _logger.info("Create item: " .. ITEM_NAME)

    local item_id = _core.add_item {
        name = ITEM_NAME,
        has_getter = false,
        has_setter = true,
        show = true,
        value_type = "dictionary.string",
        value = {},
        device_id = device_id
    }

    if item_id then
        _logger.debug("Created item ID: " .. item_id)
    else
        _logger.error("Failed to create item")
    end

    return item_id
end

local function SetItemValue (device_id, value)
    local _network = require("network")
    local setting_ids = _core.get_setting_ids_by_device_id(device_id) or {}
    local setting

    for _, setting_id in ipairs(setting_ids) do
        local _setting = _core.get_setting(setting_id)

        if _setting.info.name == RELATED_SETTING_NAME then
            setting = _setting
            break
        end
    end 

    if not value.port or (not value.ip_address and setting.value == false) then
        _logger.error("There is no port or ip address. Cannot continue.")

        return
    end

    local handle = _network.udp()
    _G.ezlo_protocols[tostring(handle)] = "udp"

    if setting.value == true then
        _network.setoption(handle, "broadcast")
        value.ip_address = "255.255.255.255"
    end

    local success, errmsg = pcall(_network.sendto,
        handle,
        value.data or "",
        value.ip_address, 
        value.port
    )

    if not success then
        _logger.error("Failed to send data, error: " .. errmsg)
    end
    
    _network.close(handle)
end

-------------------------------------------------------------------
local params = ... or {}

if not params.device_id then
    local _logger = require("logging")
    _logger.error("Missing device_id. Cannot continue.")

    return
end

if params.action == "create" then

    return CreateItem(params.device_id)

elseif params.action == "set" then

    local udp_request_cooldown = (loadfile("HUB:ezlo_protocols/scripts/functions/get_setting_by_name"){
        device_id = params.device_id,
        setting_name = "udp_request_cooldown"
    } or {}).value or 1

    local time_elapsed = os.time() - _G.ezlo_protocols.upd_last_request_time

    if time_elapsed > udp_request_cooldown then
        _G.ezlo_protocols.upd_last_request_time = os.time()
        return SetItemValue(params.device_id, params.value)
    end
    
end
