local ITEM_NAME = "udp_server_receive"
local RELATED_SETTING_NAME = "udp_server_port"
local ENABLE_UDP_SERVER_SETTING_NAME = "enable_udp_server"

local function CreateItem (device_id)
    local _core = require("core")
    local _logger = require("logging")
    
    _logger.info("Create item: " .. ITEM_NAME)

    local item_id = _core.add_item {
        name = ITEM_NAME,
        has_getter = true,
        has_setter = false,
        show = true,
        value_type = "dictionary.string",
        value = {},
        device_id = device_id
    }

    if item_id then
        _logger.debug("Created item ID: " .. item_id)
    else
        _logger.error("Failed to create item")
    end

    return item_id
end

local function EnableServer(device_id)
    local _logger = require("logging")

    if _G.ezlo_protocols[ENABLE_UDP_SERVER_SETTING_NAME] then
        _logger.warning("The UDP server is already started.")

        return 
    end

    local _network = require("network")
    local _core = require("core")

    local setting_ids = _core.get_setting_ids_by_device_id(device_id) or {}
    local setting

    for _, setting_id in ipairs(setting_ids) do
        local _setting = _core.get_setting(setting_id)

        if _setting.info.name == RELATED_SETTING_NAME then
            setting = _setting
            break
        end
    end

    local handle = _network.udp()
    
    _network.setsockname(handle, "*", tostring(setting.value))
    _G.ezlo_protocols[ENABLE_UDP_SERVER_SETTING_NAME] = handle
end

local function DisableServer()
    local _network = require("network")
    local _logger = require("logging")

    _logger.debug("Stopping UDP server.")
    local success, errmsg = pcall(_network.close,_G.ezlo_protocols[ENABLE_UDP_SERVER_SETTING_NAME])

    if not success then
        _logger.error("Failed to close UDP server: " .. errmsg)
    end

    _G.ezlo_protocols[ENABLE_UDP_SERVER_SETTING_NAME] = nil
end
-------------------------------------------------------------------
local params = ... or {}

if params.action == "create" then
    return CreateItem(params.device_id)
elseif params.action == "enable" then
    EnableServer(params.device_id)
elseif params.action == "disable" then
    DisableServer()
elseif params.action == "restart" then
    DisableServer()
    EnableServer(params.device_id)
end
