local _logger = require("logging")

local SETTING_NAME = "udp_request_cooldown"

local function CreateOrUpdateSetting(device_id, setting_id, value)

    value = value or 1

    if setting_id then
        local _core = require("core")
        _logger.debug("Update setting value: " .. SETTING_NAME .. " = " .. tostring(value))
        _core.set_setting_value(setting_id, value, "synced")
        return
    end

    local _setting_utils = require("HUB:ezlo_protocols/scripts/modules/setting_utils")

    local setting = {
        label = {
            text = "UDP request cooldown"
        },
        description = {
            text = "Set UDP request cooldown in seconds."
        },
        value_type = "int",
        value_min = 1,
        value_max = 100,
        value_default = 1,
        value = value,
        status = "synced",
        has_setter = true,
        info = {
            name = SETTING_NAME,
        }
    }

    _setting_utils.CreateSetting(device_id, SETTING_NAME, setting)
end

local params = ... or {}

if not params.device_id then
    _logger.error("Missing device_id. Cannot continue.")

    return
end

CreateOrUpdateSetting(params.device_id, params.setting_id, params.value)
